package net.eduvax.util;

import java.util.Collection;
import java.util.Iterator;

public class SortedList<T extends Comparable> {
	private class Node<T> {
		T elem;
		Node<T> next;
		Node<T> prev;
		public Node(T e,Node<T> p,Node<T> n) {
			elem=e;
			prev=p;
			next=n;
		}
	}
	private Node<T> _head=null;
	private Node<T> _tail=null;
	private int _count=0;
 	public int size() {
		return _count;
	}
	public boolean isEmpty() {
		return _count==0;
	}
	public SortedList() {
	}
	public boolean add(T e) {
		Node<T> p=null;
		Node<T> node=_tail;
		while (node!=null&&node.elem.compareTo(e)>0) {
			p=node;
			node=node.prev;
		}
		Node<T> newNode=new Node<T>(e,node,p);
		if (node==_tail) {
			_tail=newNode;
		}
		if (p!=null) {
			p.prev=newNode;
		}
		if (node!=null) {
			node.next=newNode;
		}
		else {
			_head=newNode;
		}
		_count++;
		return true;
	}
	public boolean addAll(Collection<? extends T> c) {
		Iterator<? extends T> it=c.iterator();
		while (it.hasNext()) {
			add(it.next());
		}
		return true;
	}
	public void clear() {
		_count=0;
		_head=null;
		_tail=null;
	}
	public boolean equals(Object o) {
		SortedList l=(SortedList)o;
		return l._head==_head && l._tail==_tail;
	}
	public boolean contains(Object o) {
		Node node=_head;
		boolean found=false;
		while (node!=null&&!found) {
			found=node.elem==o;
		}
		return found;
	}
	public boolean containsAll(Collection<?> c) {
		Iterator it=c.iterator();
		boolean res=true;
		while (it.hasNext() && res) {
			res=res&&contains(it.next());
		}
		return res;
	}
	public T head() {
		return _head==null?null:_head.elem;
	}
	public T tail() {
		return _tail==null?null:_tail.elem;
	}
	public T popHead() {
		T res=null;
		if (_head!=null) {
			res=_head.elem;
			_head=_head.next;
			if (_head!=null) {
				_head.prev=null;
			}
			_count--;
		}
		return res;
	}
	public T popTail() {
		T res=null;
		if (_tail!=null) {
			res=_tail.elem;
			_tail=_tail.prev;
			_tail.next=null;
			_count--;
		}
		return res;
	}
	public T elementAt(int index) {
		T res=null;
		Node<T> node=_head;
		int i;
		for (i=1;i<index&&node!=null;i++) {
			node=node.next;	
		}
		if (i==index) {
			res=node.elem;
		}
		return res;
	}
	public boolean remove(T e) {
		Node<T> node=_head;
		while (node!=null&&node.elem!=e) {
			node=node.next;
		}
		if (node!=null) {
			_count--;
			if (node.next!=null) {
				node.next.prev=node.prev;
			}
			if (node.prev!=null) {
				node.prev.next=node.next;
			}
			if (node==_head) {
				_head=node.next;
			}
			if (node==_tail) {
				_tail=node.prev;
			}
			return true;
		}
		return false;
	}
	public T[] toArray(T[] a) {
		T[] res=a;
		if (a.length<_count) {
			res=(T[])(new Object[_count]);
		}
		Node<T> node=_head;
		for (int i=0;i<_count;i++) {
			res[i]=node.elem;
			node=node.next;
		}
		return res;
	}
}
